import uuid
import shlex
import json


def my_random_string(string_length=10):
    """Returns a random string of length string_length.

    :param string_length: a positive int value to define the random length
    :return:
    """
    a = 3
    assert string_length >= a, "No es un valor positivo sobre " + str(a)
    random = str(uuid.uuid4())  # Convert UUID format to a Python string.
    random = random.upper()  # Make all characters uppercase.
    random = random.replace("-", "")  # Remove the UUID '-'.
    return random[0:string_length]  # Return the random string.
# str(my_random_string(6))
# print(my_random_string(6)) # For example, D9E50C


def fill_pattern(var_list, pattern):
    """
    Replace values in pattern
    var_list has to have 'pattern' and 'value' keys
    pattern is a string with some keys inside
    """
    code = pattern
    for lista in var_list:
        keys = lista.keys()
        # print(lista)
        assert 'pattern' in keys and 'value' \
            in keys, "Lista incorrecta en " + str(lista)
        code = code.replace(lista['pattern'], lista['value'])
    # print(code)
    return code


def pattern_value(pattern_str, val):
    "Return a specific dictionary with keys pattern and value"
    return dict(pattern=pattern_str, value=val)


def gns_dumps(string, char='#'):
    a = json.dumps(string)
    b = a.replace('\"', char)
    return b


def gns_loads(string, char='#'):
    a = string.replace(char, '\"')
    b = json.loads(a)
    return b


def context_split(value, separator='|'):
    """
    Split and take care of \"\"
    """
    q = shlex.shlex(value, posix=True)
    q.whitespace += separator
    q.whitespace_split = True
    q.quotes = '\"'
    q_list = list(q)
    return q_list



import socket
sock = None
AF_TYPE = socket.AF_INET
SOCK_TYPE = socket.SOCK_STREAM
gnc_path = '/tmp/gnc.socket'
host=socket.gethostbyname(socket.gethostname())
port=6677
address=(host,port)
n_listen=5
t_out = 15
#bufsize: standar: 128
buffsize = 64
# msg_limit = 512
HEADER = "MSG"
ENDER = "END"
uin = 6
char_code = "utf-8"
TEST_TEXT = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Phasellus interdum suscipit mi, viverra volutpat augue elementum eget. Vivamus et ligula in nisl tincidunt vestibulum. Nullam non bibendum libero, nec vulputate enim. Cras vulputate enim vitae purus porttitor, pharetra faucibus ipsum pulvinar. In id facilisis lacus, vitae vestibulum justo. Ut a porttitor sem, eu auctor mi. Proin erat lacus, tempus tincidunt mi et, ultricies consequat turpis. Fusce fringilla feugiat nunc, in ullamcorper tellus tincidunt et. Integer pretium mauris lorem, ac vulputate sem luctus quis. Maecenas vitae sapien sit amet dolor auctor molestie eu non eros.\
In iaculis ligula sagittis, tincidunt massa quis, commodo sapien. Fusce nibh tellus, varius laoreet rhoncus sit amet, sodales at nisl. Nulla eros sapien, laoreet quis accumsan quis, volutpat id orci. Aenean non mollis magna. Aliquam blandit, felis ac hendrerit blandit, mi justo convallis nulla, non mattis neque ipsum vitae ligula. Sed dapibus eleifend tellus, non rutrum risus cursus eget. Morbi cursus molestie iaculis. Phasellus blandit facilisis sollicitudin. Suspendisse porttitor, quam ac gravida luctus, diam nunc condimentum odio, in consequat justo lacus in magna. Ut ultrices, quam a aliquet lacinia, nibh purus pretium felis, quis malesuada est elit in dui. Cras sed felis ultricies, interdum risus eu, pretium turpis. Praesent aliquet placerat orci eu sollicitudin. Maecenas convallis euismod magna, et consequat enim porttitor et. Integer bibendum velit in suscipit placerat. Aliquam dui nunc, porta sit amet convallis sit amet, finibus vel orci.\
Vivamus suscipit urna vitae mi molestie aliquet. Duis porta eu mi nec dignissim. Mauris mi velit, accumsan vitae ipsum vel, posuere faucibus dolor. Curabitur interdum elit ac malesuada porttitor. Curabitur semper risus sed magna vestibulum egestas. Duis in velit volutpat, iaculis mi semper, dapibus risus. Vivamus dui augue, sodales non dui eget, rutrum molestie arcu. In hac habitasse platea dictumst.\
Nullam a purus id ex varius consequat. Sed in pellentesque dui. Vivamus enim augue, feugiat at euismod et, vulputate vel tortor. Quisque a enim augue. Pellentesque aliquet venenatis volutpat. Praesent viverra odio a euismod tincidunt. Aenean ut sagittis eros. Praesent lacus metus, blandit iaculis maximus non, elementum et augue. Nam lorem dui, tempus quis mollis nec, consectetur ac nisl. Nulla facilisi. Nam accumsan velit libero, sit amet placerat ex ultrices ac. Sed commodo luctus eros eu tincidunt. Suspendisse varius arcu dolor, ut ullamcorper metus malesuada in. Nunc ultrices mauris neque, a vulputate ante molestie nec.\
Nunc tristique eu nunc vitae mattis. Donec tincidunt et tellus vitae pellentesque. Quisque luctus enim sed facilisis maximus. Aliquam pulvinar lectus in dui ullamcorper malesuada. Duis elementum, dui sit amet pellentesque semper, sem ligula facilisis nisi, ut vehicula mauris elit in arcu. Sed facilisis facilisis nibh, nec sagittis tellus iaculis sit amet. Vivamus ante nisl, tempor sit amet nibh in, eleifend egestas quam. Praesent nec augue vitae tortor rutrum accumsan non ut nisi. Nulla et lacus ut libero venenatis iaculis vitae sit amet nisl. as"



# Standar lib
import asyncio
import functools
from multiprocessing import Manager, Queue, Lock

# contrib modules
import ujson as json

# Own module
from .gn_socket import GNCSocket

# module tasktools
from tasktools.taskloop import coromask, renew, simple_fargs
from networktools.colorprint import gprint, bprint, rprint

from networktools.library import pattern_value, \
    fill_pattern, context_split, \
    gns_loads, gns_dumps

tsleep = 2


class GNSocketServer:

    def __init__(self, *args, **kwargs):
        self.address = kwargs.get('address', ('localhost', 6666))
        # This queue list is fixed and allow us to control
        # the scheduler system with more or less instances
        # not to send msg to every instace
        self.common_queues = kwargs.get('common_queues', {})
        self.gs = GNCSocket(mode='server')
        self.gs.set_address(self.address)
        self.rdb_status = False

    async def read_queue(self, queue):
        for i in range(queue.qsize()):
            msg_in = queue.get()
            msg = msg_in['msg']
            idc = msg_in['idc']
            await self.gs.send_msg(json.dumps(msg), idc)

    async def sock_write(self, gs, queue_t2n, idc):
        # Receive from sources and send data to clients
        rprint("New sock_write iteration")
        gs = self.gs
        queue = queue_t2n
        await asyncio.sleep(tsleep)
        try:
            print("Checking queues>qt2n? <", queue.empty(), "> on:", queue)
            if not queue.empty():
                # read control queues
                for i in range(queue.qsize()):
                    msg_in = queue.get()
                    queue.task_done()
                    msg = msg_in.get('msg')
                    idc = msg_in.get('idc')
                    bprint("()()()")
                    print("XDX Enviando mensaje: %s" % msg_in, flush=True)
                    bprint("()()()")
                    await gs.send_msg(json.dumps(msg), idc)
                # read data queues
            for ids, channel in self.common_queues.items():
                table_writer = channel.get('writer')
                # get from writer table
                cursor = await self.read_msg(table_writer)
                # gprint("#########################")
                #rprint("Ready to send by socket to user client %s" % cursor)
                # gprint("#########################")
                if cursor:
                    for c in cursor:
                        msg = c.get('msg', {})
                        bprint("Enviando a cliente user interface %s" % msg)
                        # What are we sending? --> check the test for server
                        # await gs.send_msg(json.dumps(msg), idc)
                        await gs.send_msg(json.dumps(c), idc)
        except Exception as ex:
            gs.logger.exception("Error con modulo de escritura del socket %s" %ex)
            print("Error con modulo de escritura del socket")
            raise ex

    # socket communication terminal to engine
    async def sock_read(self, gs, queue_n2t, idc):
        # read from client and send to the manager
        # the datagrams must bring the source id: ids
        loop = asyncio.get_event_loop()
        queue = queue_n2t
        msg_from_engine = []
        await asyncio.sleep(tsleep)
        try:
            # bprint("XDX Check socket from gui", flush = True)
            # read queue is answer from msg sended
            datagram = await gs.recv_msg(idc)
            print("=============")
            print(datagram)
            msg_dict = {}
            if not datagram == '' and \
               datagram != "<END>":
                msg_dict = json.loads(datagram)
                # CHECK IF A COMMAND FOR SYSTEM
                # 'SYSTEM' in msg_dict
                bprint("====SOCK READ ON GNSOCKET SERVER ===")
                rprint(msg_dict)
                destiny = msg_dict.get('msg')[0].get('type', None)
                print("DESTINIY:::::")
                bprint(destiny)
                await asyncio.sleep(5)
                # rprint(datagram)
                # bprint(type(datagram))
                # bprint(msg_dict)
                #print("MSG Type on sock %s" % msg_dict.get('msg')[0].get('type', None))
                if destiny == 'SYSTEM':
                    # first queue sends....
                    #rprint("Sends grom network to term")
                    # gprint("=====")
                    new_dict = {
                        'dt': msg_dict,
                        'idc': idc
                    }
                    bprint("Sending data by queue to sock--->write_sock on", queue)
                    print("New dict to n2t")
                    bprint(new_dict)
                    queue.put(new_dict)
                    queue.join()
                elif destiny == 'DESTINY':
                    # print(msg_dict)
                    # print(type(msg_dict))
                    msg_dict['traces'].update({'idc_server': idc})
                    # get ids if exists or five the first ids from common_queues
                    ids = msg_dict.get('msg')[0].get(
                        'args')[0].get('ids', None)
                    #print("IDS on sock %s" % ids)
                    if ids:
                        """
                        Then the msg_dict is a correct dict with
                        the id of the source
                        """
                        #print("Sendind msg to service %s "%msg_dict)
                        # bprint(self.common_queues)
                        channel = self.common_queues.get(
                            ids, {'reader': 'interface'})
                        # print(channel)
                        table_reader = channel.get('reader')
                        # Put on the list to send to the source
                        # send msg with the traces and msg, mgs has keys ['type','command','args']
                        await self.write_msg(table_reader, msg_dict)
        except Exception as ex:
            gs.logger.exception("Error con modulo de lectura del socket %s" %ex)            
            print("Error con modulo escritura socket", ex)
            raise ex

    def socket_task(self):
        #print("XDX socket loop inside", flush=True)
        with GNCSocket(mode='server') as gs:
            #gs = GNCSocket(mode='server')
            loop = asyncio.get_event_loop()
            self.loop = loop
            gs = self.gs
            queue_list = self.queue_list
            self.active_session()
            gs.set_address(self.address)
            gs.set_loop(loop)
            try:
                async def socket_io(reader, writer):
                    print("Entrando a socket -io")
                    queue_read = queue_list[0]
                    queue_write = queue_list[1]
                    idc = await gs.set_reader_writer(reader, writer)
                    # First time welcome
                    welcome = json.dumps({"msg": "Welcome to socket"})
                    print("Enviando msg welcome--%s" % welcome)
                    await gs.send_msg(welcome, idc)
                    await asyncio.sleep(0.1)
                    if not self.rdb_status:
                        await self.init_async(loop)
                        if self.conn:
                            print("Rethinkdb connected")
                            self.rdb_status = True
                    # task reader
                    try:
                        args = [gs, queue_read, idc]
                        task_1 = loop.create_task(
                            coromask(
                                self.sock_read,
                                args,
                                simple_fargs)
                        )
                        task_1.add_done_callback(
                            functools.partial(
                                renew,
                                task_1,
                                self.sock_read,
                                simple_fargs)
                        )
                        args = [gs, queue_write, idc]
                        # task write
                        task_2 = loop.create_task(
                            coromask(
                                self.sock_write,
                                args,
                                simple_fargs)
                        )
                        task_1.add_done_callback(
                            functools.partial(
                                renew,
                                task_1,
                                self.sock_write,
                                simple_fargs)
                        )
                    except Exception as exe:
                        gs.logger.exception("Error con modulo de escritura del socket %s" %ex)                        
                        print("Execepcion al levantar servicio", exe)
                        gs.close()
                        if not self.conn:
                            print("Clossing RDB")
                            self.conn.close()
                        raise exec
                gprint("=")
                gprint("loop"+str(loop))
                rprint("Loop is runnign?<"+str(loop.is_running())+">")
                print("Creating socket server future")
                future = loop.create_task(
                    gs.create_server(socket_io, loop))
                rprint(loop.is_running())
                gprint("=")
                print("Future de server socket->")
                print(future)
                print(loop.is_running())
                if not loop.is_running():
                    rprint("listening clients......-----=====")
                    print(self.address)
                    loop.run_forever()
                    print(future)
                else:
                    print("Running Future")
                    loop.run_until_complete(future)
            except KeyboardInterrupt:
                print("Closing socket on server")
                gs.close()
                print("Doing wait_closed")
                loop.run_until_complete(gs.wait_closed())
            except Exception as ex:
                print("Otra exception", ex)
            finally:
                print("Clossing loop on server")
                # loop.close()

        async def read_msg(self, writer, *args):
            """
            Must be implemented
            """
            return []

        async def write_msg(self, reader, *args):
            """
            Must be implemented
            """
            pass

        def active_session(self):
            pass

        async def init_async(self, loop):
            pass

        def set_reader(self, fn_reader):
            # To use when subclass
            self.read_msg = fn_reader

        def set_writer(self, fn_writer):
            # To use when subclass
            self.write_msg = fn_writer

        def close(self):
            self.gs.close()



import asyncio
import socket
from basic_queuetools.queue import read_queue_gen, read_async_queue
# contrib modules
import ujson as json

# Own module
from gnsocket.exceptions import clean_exception

# module tasktools
from networktools.colorprint import gprint, bprint, rprint


tsleep = 2


class GNCSocketBase:

    def __init__(self, queue_n2t,
                 queue_t2n,
                 mode,
                 callback_exception=clean_exception,
                 *args,
                 **kwargs):
        self.qn2t = queue_n2t
        self.qt2n = queue_t2n
        self.address = kwargs.get('address', ('localhost', 6666))
        self.log_path = kwargs.get('log_path', '~/gnsocket_log')
        self.mode = mode
        self.exception = callback_exception
        self.timeout = kwargs.get("timeout", 20)
        self.raise_timeout = kwargs.get("raise_timeout", False)
        self.client = None
        self.client_name = kwargs.get('client_name', "test")
        self.commands = {
        }
        self.stop_read = {}
        self.stop_write = {}
        self.queues = {}
        self.clients = {}

    def new_client(self, value, name, socket_id, **kwargs):
        if name not in self.clients:
            self.clients[name] = dict()
            self.clients[name].update({
                socket_id: value
            })
            gprint(f"Nuevos clientes ::: {self.clients}")
        else:
            self.clients[name].update({
                socket_id: value
            })
            # estable relacion entre id en server
            # e id cliente

    def queue_channel(self, idc):
        q_t2n = asyncio.Queue()
        q_n2t = asyncio.Queue()
        self.queues[idc] = {'t2n': q_t2n, 'n2t': q_n2t}

    def get_channel(self, idc):
        channel = self.queues.get(idc, {})
        return channel

    async def sock_write(self, gs, idc, *args, **kwargs):
        if not idc:
            idc_list = list(gs.clients.keys())
            if idc_list:
                idc = idc_list.pop()
        await asyncio.sleep(1)
        if idc in list(gs.clients.keys()):
            queue_n2t = self.get_channel(idc).get("n2t")
            if not self.stop_write[idc]:
                try:
                    async for msg in read_async_queue(queue_n2t, fn_name='sock_write'):
                        msg_send = json.dumps(msg)
                        idc_server = msg.get('idc_server')
                        try:
                            send_msg = gs.send_msg(msg_send, idc)
                            await send_msg
                        except BrokenPipeError as be:
                            gs.report(
                                "sock_write",
                                "Close->Broken Pipe Error al cerrar %s bytes" % (be))
                            gs.logger.exception(
                                "Tiempo fuera en escritura %s, mode %s" % (
                                    be, gs.mode))
                            await asyncio.sleep(1)
                            continue
                        except socket.error as se:
                            gs.report(
                                "socke_write", "Close->Socket Error al leer %s bytes" % (se))
                            await asyncio.sleep(1)
                            continue
                        except asyncio.TimeoutError as te:
                            gs.report("sock write", "Timeout error A", te)
                            gs.logger.exception("Tiempo fuera en escritura %s, mode %s" % (
                                te, gs.mode))
                            await asyncio.sleep(1)
                            continue
                        except (ConnectionResetError, ConnectionAbortedError) as conn_error:
                            gs.report("sock write", "Error de conexion",
                                      conn_error, gs.mode)
                            gs.logger.exception("Excepción por desconexión %s, mode %s" % (
                                conn_error, gs.mode))
                            await asyncio.sleep(1)
                            gs.set_status(False)
                            del gs.clients[idc]
                            continue
                        except Exception as ex:
                            gs.on_new_client(self.client_name)
                            gs.report("sock write", "Exepción", ex)
                            gs.logger.exception(
                                "Error con modulo cliente gnsocket coro write %s" % ex)
                            print(
                                "Error con modulo de escritura del socket IDC %s" % idc)
                            continue
                except asyncio.TimeoutError as te:
                    gs.report("sock write", "Timeout error B", te)
                    gs.logger.exception("HEARTBEAT: Tiempo fuera en escritura %s, mode %s" % (
                        te, gs.mode))
                    await asyncio.sleep(10)
                except (ConnectionResetError, ConnectionAbortedError) as conn_error:
                    gs.report("sock write", "Error de conexion",
                              conn_error, gs.mode)
                    gs.logger.exception("HEARTBEAT: Excepción por desconexión %s, mode %s" % (
                        conn_error, gs.mode))
                    await asyncio.sleep(10)
                    gs.set_status(False)
                    del gs.clients[idc]
                except Exception as ex:
                    gs.report("sock write", "Exepción", ex)
                    gs.logger.exception(
                        "HEARTBEAT: Error con modulo cliente gnsocket coro write %s" % ex)
                    gs.report(
                        "sock_write",
                        f"Error con modulo de escritura del socket IDC {idc}")
            kwargs["stop"] = self.stop_write[idc]
        else:
            await asyncio.sleep(2)
        return [gs, idc, *args], kwargs
    # socket communication terminal to engine

    async def sock_read(self, gs, idc, *args, **kwargs):
        if not idc:
            idc_list = list(gs.clients.keys())
            if idc_list:
                idc = idc_list.pop()
        #queue_t2n = self.qt2n
        await asyncio.sleep(1)
        if idc in list(gs.clients.keys()):
            queue_t2n = self.get_channel(idc).get("t2n")
            try:
                recv_msg = gs.recv_msg(idc)
                datagram = await recv_msg
                if datagram not in {'', "<END>", 'null', None}:
                    msg_dict = json.loads(datagram)
                    if 'SOCKET_COMMAND' in msg_dict:
                        msg_dict['socket_id'] = idc
                        self.check_msg(msg_dict)
                    else:
                        msg = {'dt': msg_dict, 'idc': idc}
                        await queue_t2n.put(msg)
            except BrokenPipeError as be:
                gs.report(
                    "sock_read",
                    f"Close->Broken Pipe Error al cerrar {be} bytes")
                gs.logger.exception("Tiempo fuera en escritura %s, mode %s" % (
                    be, gs.mode))
                await asyncio.sleep(1)
            except socket.error as se:
                gs.report("socke_read",
                          f"Close->Socket Error al leer {se} bytes")
                await asyncio.sleep(1)
            except asyncio.TimeoutError as te:
                gs.report("sock_read", "Timeout error A", te)
                gs.logger.exception("Tiempo fuera en escritura %s, mode %s" % (
                    te, gs.mode))
                await asyncio.sleep(2)
            except (ConnectionResetError,
                    ConnectionAbortedError) as conn_error:
                gs.report("sock_read", "Error de conexion",
                          conn_error, gs.mode)
                gs.logger.exception("Excepción por desconexión %s, mode %s" % (
                    conn_error, gs.mode))
                await asyncio.sleep(2)
                gs.set_status(False)
                del gs.clients[idc]
                if self.exception:
                    self.exception(conn_error, gs, idc)
            except Exception as ex:
                gs.report("sock_read", "Exepción", ex)
                rprint(ex)
                gs.logger.exception(
                    f"Error con modulo cliente gnsocket coro read {ex}")
                gs.report("sock_read", f"Some error {ex} en sock_read")
        else:
            await asyncio.sleep(2)
            idc = self.client
        return [gs, idc, *args], kwargs

    async def from_socket(self, *args, **kwargs):
        for idc, channel in self.queues.items():
            # read queue multiplexer
            queue_t2n = channel.get('t2n')
            if not queue_t2n.empty():
                # block until process the list
                for i in range(queue_t2n.qsize()):
                    msg = await queue_t2n.get()
                    self.qt2n.put(msg)
        await asyncio.sleep(.1)
        return args, kwargs

    async def to_socket(self, *args, **kwargs):
        queue_n2t = self.qn2t
        if self.clients:
            # dicts->keys()->last value
            default = list(list(self.clients.values()).pop().keys())[-1]
            if not queue_n2t.empty():
                # block until process the list
                for i in range(queue_n2t.qsize()):
                    msg = queue_n2t.get()
                    idc = msg.get('idc', default)
                    q_n2t = self.get_channel(idc).get('n2t')
                    await q_n2t.put(msg)
        await asyncio.sleep(.1)
        return args, kwargs

    def check_msg(self, msg):
        command = msg.get('SOCKET_COMMAND', "print")
        callback = self.commands.get(command, print)
        callback(**msg)

    def set_socket_task(self, callback_socket_task):
        self.socket_task = callback_socket_task



import asyncio
from tasktools.taskloop import coromask, renew, simple_fargs


class AbstractSocketInterface:
    def __init__(self, gsocket, *args, **kwargs):
        self.gs = gsocket
        self.queue_g2s = kwargs.get('queue_g2s')
        self.queue_s2g = kwargs.get('queue_s2g')

    async def sock_write(self, *args, **kwargs):
        pass

    async def sock_read(self, *args, **kwargs):
        pass


class SocketInterface:
    def __init__(self, gsocket, *args, **kwargs):
        self.gs = gsocket
        self.queue_g2s = kwargs.get('queue_g2s')
        self.queue_s2g = kwargs.get('queue_s2g')

    async def sock_write(self, idc):
        gs = self.gs
        queue = self.queue_g2s
        await asyncio.sleep(tsleep)
        try:
            print("Check queue from GUI is empty")
            print(queue)
            print(queue.empty())
            if not queue.empty():
                for i in range(queue.qsize()):
                    msg_in = queue.get()
                    bprint("Recibido desde gui : %s" % msg_in)
                    await gs.send_msg(gns_dumps(msg_in), idc)
            else:
                pass
        except Exception as exec:
            print("Error con modulo de escritura del socket")
            raise exec
    # socket communication terminal to engine

    async def sock_read(self, idc):
        gs = self.gs
        queue = self.queue_s2g
        msg_from_engine = []
        await asyncio.sleep(tsleep)
        try:
            print("Check if there are messages from source")
            # read queue is answer from msg sended
            datagram = await gs.recv_msg(idc)
            bprint("Recibido desde socket server %s " % datagram)
            if not datagram == '' and \
               datagram != "<END>":
                queue.put(gns_loads(datagram))
        except Exception as exec:
            raise exec

    def socket_task(queue_list):
        # client socket
        print("Socket tasks: (writer, reader)")
        loop = asyncio.get_event_loop()
        gs.set_loop(loop)

        async def socket_io(queue_list):
            queue_read = queue_list[0]
            queue_write = queue_list[1]
            idc = await gs.create_client()
            # First time welcome
            #welcome={'command':'greetings', 'args':["Welcome to socket"]}
            # rprint(welcome)
            # await gs.send_msg(welcome)
            # await gs.send_msg("<END>")
            # task reader
            try:
                args = [queue_read, idc]
                task_1 = loop.create_task(
                    coromask(
                        sock_read,
                        args,
                        simple_fargs)
                )
                task_1.add_done_callback(
                    functools.partial(
                        renew,
                        task,
                        sock_read,
                        simple_fargs)
                )
                args = [queue_write, idc]
                # task write
                task_2 = loop.create_task(
                    coromask(
                        sock_write,
                        args,
                        simple_fargs)
                )
                task_2.add_done_callback(
                    functools.partial(
                        renew,
                        task,
                        sock_write,
                        simple_fargs)
                )
            except Exception as exec:
                raise exec

        ########
        # Insert a coroutine with reader and writer tasks

        async def activate_sock(queue_list):
            await socket_io(queue_list)
            return "socket loaded"

        future1 = loop.create_task(activate_sock(queue_list))

        print("Loop is running?", loop.is_running())
        if not loop.is_running():
            loop.run_forever()



# -*- coding: utf-8 -*-

# Echo server program
import os
import socket
import asyncio
import errno
import math
import re
import ssl
from pathlib import Path
from basic_logtools.filelog import LogFile
from asyncio import shield, wait_for, Task
# These values are constant
try:
    from conf.socket_conf import AF_TYPE, SOCK_TYPE
    from conf.socket_conf import HEADER
    from conf.socket_conf import ENDER
    from conf.socket_conf import sock
    from conf.socket_conf import gnc_path
    from conf.socket_conf import t_out
    from conf.socket_conf import buffsize
    from conf.socket_conf import uin
    from conf.socket_conf import char_code
    from conf.socket_conf import n_listen
    from conf.socket_conf import address

except Exception:
    from .conf.socket_conf import AF_TYPE, SOCK_TYPE
    from .conf.socket_conf import HEADER
    from .conf.socket_conf import ENDER
    from .conf.socket_conf import sock
    from .conf.socket_conf import gnc_path
    from .conf.socket_conf import t_out
    from .conf.socket_conf import buffsize
    from .conf.socket_conf import uin
    from .conf.socket_conf import char_code
    from .conf.socket_conf import n_listen
    from .conf.socket_conf import address

from networktools.library import (pattern_value,
                                  fill_pattern,
                                  context_split,
                                  gns_loads,
                                  gns_dumps,
                                  my_random_string,
                                  complete_nro,
                                  hextring,
                                  hextring2int)

from networktools.colorprint import gprint, bprint, rprint
from networktools.time import now, timestamp
from networktools.path import home_path

# Asyncio guide:
# http://www.snarky.ca/how-the-heck-does-async-await-work-in-python-3-5

# More ref about socket:
# http://stackoverflow.com/questions/27014955/socket-connect-vs-bind

# example:

# Best example: https://gist.github.com/jamilatta/7603968

# from scheduler import ReadWait, WriteWait

import uuid

import queue
import chardet
from datetime import datetime, timedelta


class GNCSocket:
    """
    This class allows yo to enable a socket to communicate among different process or terminals.

    Define a protocol that take a string of bytes and slice in an amount of parts related to the
    *buffsize* parameter.

    You can select the *mode* to use this class: like a server to *listen* connections or like a client to *connect* to some source.

    Also, if you need security, set the ssl parameter to True.

    The *backlog* parameter aludes to the queued connections to the server.


    Socket class for GNC
    Create or connect to an UNIX/TCP socket
    Methods:
    * connect
    * send_msg
    * recv_msg
    * server
    * client
    """

    def __init__(self, mode='server', address=address, **kwargs):
        self.mode = mode
        self.bs = kwargs.get("buffsize", buffsize)
        self.gnc_path = kwargs.get("gnc_path", gnc_path)
        # default timeout
        self.timeout = kwargs.get('timeout', t_out)
        self.raise_timeout = kwargs.get("raise_timeout", False)
        self.module = kwargs.get('module', 'test')
        self.ssl = kwargs.get('ssl', False)
        # IF SSL ACTIVE
        if self.ssl:
            self.set_ssl(**kwargs)
        else:
            self.context = None
        ####
        self.address = kwargs.get('address', address)
        self.AF_TYPE = kwargs.get('AF_TYPE', AF_TYPE)
        # self.set_logger()
        if mode == 'server':
            self.set_server(self.AF_TYPE, self.address)
            self.server = None
        if mode == 'client':
            self.set_client(self.AF_TYPE, self.address)
        self.status = False
        # BIND: connect a path with socket
        # list of connections, address.
        self.backlog = kwargs.get("backlog", n_listen)
        # MSG Format
        # Defined:
        # LEN_HEAD MSG LCHAR(hex) LEN MSG END
        # Send format
        self.msg_struct = b"IDX hex(PAGE)/hex(TOT_PAGES) MSG hex(LEN) MSG END"
        self.msg_template = "{idx} {page}/{tot_pages} MSG {length} MSG END"
        self.msg_spaces = [x.start()
                           for x in re.finditer(b' ', self.msg_struct)]
        # IDX hex(PAGE)/hex(TOT_PAGES) MSG hex(LEN) MSG END
        # How to generate:
        # msg -> bytes
        # len(msg_bytes)
        # # envios
        # self.msg_limit = msg_limit
        # self.msg_limit_hex = str(hex(self.msg_limit)).split('x')[1]
        # self.nchar = len(self.msg_limit_hex)
        # Limit_msg = 512 (default), n=len(str(512))
        # To hex: 0x200
        # Split x
        # Get value and complete => LCHAR (en hex)
        self.header = kwargs.get("header", HEADER)
        self.ender = kwargs.get("ender", ENDER)
        # number of chars for id
        self.uin = kwargs.get("uin", uin)
        self.idx = []
        self.conns = []
        self.addrs = []
        self.msg_r = ''
        # asyncio coroutines
        self.alert = ""
        self.loop = ''
        self.mq = {}
        # message queues
        self.clients = {}
        self.idc = []
        self.server = object
        # a fn to print or show on some other screen
        self.report = kwargs.get("report", print)
        #
        # new client queue: if a new client is created, then give
        # the idc value to heart_beat assigned to this client
        self.new_client_queue = asyncio.Queue()
        #
        log_path = home_path(kwargs.get('log_path', '~/socket_log'))
        log_level = kwargs.get('log_level', 'INFO')
        self.logger = LogFile(self.class_name,
                              self.mode,
                              "_".join(map(str, self.address)),
                              path=log_path,
                              base_level=log_level)

        self.queue_socket_status = None
        self.active_conn = {}
        self.relation = {}
        self.closing = {}
        self.reading = {}
        self.queue_cancel = queue.Queue()
        self.busy_read = False

    def off_new_client(self, key):
        if key in self.active_conn.keys():
            self.active_conn[key] = False

    def on_new_client(self, key):
        if key in self.active_conn.keys():
            self.active_conn[key] = True

    def drop_conn(self, key):
        if key in self.active_conn:
            del self.active_conn[key]

    def new_client(self, key):
        return self.active_conn.get(key)

    def set_new_client(self, key):
        self.active_conn[key] = True

    @property
    def class_name(self):
        return self.__class__.__name__

    def set_queue_socket_status(self, queue):
        self.queue_socket_status = queue

    def set_ssl(self, **kwargs):
        purpose_key = kwargs.get('ssl_protocol', 'tls')
        self.protocols = {
            'tls': ssl.PROTOCOL_SSLv23,
        }
        self.protocol = self.protocols.get(purpose_key, ssl.PROTOCOL_SSLv23)
        self.set_certs = kwargs.get('set_certificates',
                                    {'cafile': None,
                                     'capath': None,
                                     'cadata': None,
                                     'crtkey': None})
        data_ssl = {}
        data_ssl.update(self.set_certs)
        self.ssl_context(self.protocol, data_ssl)

    def ssl_context(self, protocol, data_ssl):
        self.context = ssl.SSLContext(protocol)
        crt = data_ssl.get('cafile')
        key = data_ssl.get('crtkey')
        if self.mode == 'server':
            self.context.load_cert_chain(crt, key)
        elif self.mode == 'client':
            path = data_ssl.get('cadata')
            data = data_ssl.get('capath')
            self.context.load_verify_locations(crt, path, data)
        print("SSL Settings end")

    def send_msg_sock_status(self, value):
        msg = {'command': 'socket_bar',
               'status': value}
        if self.queue_socket_status:
            self.queue_socket_status.put(msg)
            self.queue_socket_status.join()

    def set_server(self, AF_TYPE, address):
        if AF_TYPE == socket.AF_UNIX:
            self.address = self.gnc_path
            if os.path.exists(self.gnc_path):
                os.remove(self.gnc_path)
        elif AF_TYPE == socket.AF_INET:
            self.address = address
        # self.logger.info("Se crea un socket server")

    def set_client(self, AF_TYPE, address):
        if AF_TYPE == socket.AF_UNIX:
            self.address = self.gnc_path
        elif AF_TYPE == socket.AF_INET:
            self.address = address
        # self.logger.info("Se crea un socket client")

    def set_status(self, value):
        if value in [True, False]:
            self.status = value
        else:
            print("Status don't change")
        # self.logger.info("Se crea modifica status a %s" %value)

    def switch_status(self):
        self.status = not self.status
        # self.logger.info("Se crea modifica status a %s" %self.status)

    def get_status(self):
        return self.status

    def set_loop(self, loop):
        print(format(loop))
        self.loop = loop

    def get_writer(self, idc):
        return self.clients[idc]['writer']

    def get_reader(self, idc):
        return self.clients[idc]['reader']

    def generate_msg(self, msg: str):
        # self.msg_struct = b"IDX hex(PAGE)/hex(TOT_PAGES) MSG hex(LEN) MSG END"
        # self.msg_tempalte = b"{header} {page}/{tot_pages} MSG {length} MSG END"
        b_msg = msg.encode(char_code)
        T_LEN = len(b_msg)
        # Cantidad de caracteres en hexadecimal
        hex_T_LEN = hextring(T_LEN)
        # Se obtiene el largo de caracteres como
        # cota superior de largo de mensaje
        # A utilizar en paginacion
        # largo del valor
        # n_char es lo mismo
        new_n = len(hex_T_LEN)
        self.n_char = new_n
        # obtener el largo base del mensaje
        self.base_len = len(self.header) + len(self.ender) + \
            len(self.msg_spaces) + self.uin + 3 * self.n_char + 1
        # n_char is the amount of chars for numbers
        # tha last  +1 is for the space after IDX
        # Largo de pedazo de mensaje a enviar por partes
        self.len_msg = self.bs - self.base_len

        assert self.len_msg >= 1, "Debes definir un buffsize de mayor tamaño"
        assert new_n + 1 < self.len_msg, "No es posible enviar mensaje"

        # Cantidad maxima de mensajes a enviar
        n_msgs = math.ceil(T_LEN / self.len_msg)
        # transformar a hex y sacar string
        hex_n_msgs = hextring(n_msgs)
        # Cantidad de paginas
        self.N_n_msgs = len(str(hex_n_msgs))
        # hex_nmsgs = str(hex(n_msgs)).split('x')[1]
        ender = self.ender.encode(char_code)
        # Se construye: hex(PAGE)/hex(TOT_PAGES) MSG hex(LEN) MSG END
        NON = "".encode(char_code)
        head_template = "{header} {page}/{tot_pages} {length} "
        msg_ender = " END"
        if n_msgs > 1:
            for i in range(n_msgs):
                # Construir header:
                # Conocer parte de msg a enviar
                # Conocer largo de msg_i
                # Cantidad carácteres largo
                step = i * self.len_msg
                this_page = hextring(i + 1)
                msg_i = b_msg[step:step + self.len_msg]
                msg_l = len(msg_i)
                # +2 erased because quoted don't go
                msg_len = msg_l + len(ender)
                # print("MSG to send -> Encoded by" , the_encoding)
                componentes = {
                    'header': self.header,
                    'page': complete_nro(this_page, n=new_n),
                    'tot_pages': complete_nro(hex_n_msgs, n=new_n),
                    'length': complete_nro(hextring(msg_len), n=new_n)
                }
                msg_header = head_template.format(**componentes)
                this_msg = NON.join([msg_header.encode(char_code),
                                     msg_i,
                                     msg_ender.encode(char_code)])
                #bprint("Generated msg Encoded by")
                # rprint(this_msg)
                yield this_msg

        elif n_msgs == 1:
            this_page = hextring(1)
            msg_len = (T_LEN + len(ender))
            hex_msg_len = complete_nro(hextring(msg_len), n=new_n)
            componentes = {
                'header': self.header,
                'page': complete_nro(this_page, n=new_n),
                'tot_pages': complete_nro(hextring(1), n=new_n),
                'length': complete_nro(hextring(msg_len), n=new_n)
            }
            msg_header = head_template.format(**componentes)
            this_msg = NON.join([msg_header.encode(char_code),
                                 b_msg,
                                 msg_ender.encode(char_code)])
            #print("Generated msg Encoded by" , the_encoding)
            yield this_msg

    def gen_idx(self):
        IDX = my_random_string(self.uin)
        t = True
        while t:
            if IDX not in self.idx:
                self.idx.append(IDX)
                t = False
            else:
                IDX = my_random_string(self.uin)
        return IDX

    async def send_msg(self, msg, id_client):
        writer = self.clients.get(id_client).get('writer')
        # tot = self.N_n_msgs
        try:
            if writer.transport.is_closing():
                # self.logger.error("La conexión se cerró %s" % self.status)
                raise Exception("Conexión perdida")
            # assert tot == len(msg), "Hay un cruce de mensajes"
            pre = datetime.now()
            await shield(self.send_text(msg, writer))
            delta = datetime.now()-pre
            await shield(self.send_text('<END>', writer))
        except asyncio.CancelledError as ex:
            bprint("Cancelled error en send_msg")
            self.logger.exception(
                "Hubo una excepción, error de cancelación con modulo asyncio %s" % self.status)
            raise ex
        except (ConnectionResetError, ConnectionAbortedError) as conn_error:
            rprint("Error de conexion")
            self.logger.exception("Excepción por desconexión %s, mode %s" % (
                conn_error, self.mode))
            await asyncio.sleep(10)
            raise conn_error
        except socket.error as ex:
            bprint("Socket error en send msg")
            self.logger.exception(
                "Hubo una excepción, error en socket %s" % self.status)
            await asyncio.sleep(10)
            raise ex
        except Exception as ex:
            bprint("Excpecion desconocida send_msg")
            self.set_status(False)
            await asyncio.sleep(10)
            self.logger.exception("Hubo una excepción %s" % self.status)
            raise ex
        except asyncio.TimeoutError as te:
            bprint("Send-msg timeouterror")
            self.logger.exception(
                "Hubo una excepción, error de timeout con modulo asyncio %s" % te)
            raise te

    async def send_text(self, msg, writer):
        IDX = self.gen_idx().encode(char_code)
        for b_msg in self.generate_msg(msg):
            # print(b_msg)
            # Here the splited messages
            # First value is header length
            # Catch them
            # 1+1+self.uin
            # Find first space to get index from header len value
            to_send = b"".join([IDX, b" ", b_msg])
            # print("Contexted:::>",q)
            # yield WriteWait(conn)
            if writer.get_extra_info('peername'):
                writer.write(to_send)
                # Transport.write(data)
                await writer.drain()
            else:
                writer.write_eof()
                await writer.close()

    def get_extra_info(self, idc):
        writer = self.clients.get(idc).get('writer')
        return writer.get_extra_info('peername')

    def send_eof(self, id_client):
        writer = self.clients.get(id_client).get('writer')
        writer.write_eof()

    def abort(self, id_client):
        writer = self.clients.get(id_client).get('writer')
        writer.abort()

    def at_eof(self, id_client):
        reader = self.clients.get(id_client).get('reader')
        return reader.at_eof()

    def feed_eof(self, id_client):
        reader = self.clients.get(id_client).get('reader')
        reader.feed_eof()

    async def heart_beat(self, idc, *args, **kwargs):
        sleep = kwargs.get('sleep', 2)
        await asyncio.sleep(sleep)
        client_name = kwargs.get('client_name')
        tnow = now()
        if client_name:
            idc = self.relation.get(client_name)
        reader = self.clients.get(idc, {}).get('reader')
        writer = self.clients.get(idc, {}).get('writer')
        if idc in self.clients.keys() and reader and writer:
            await asyncio.sleep(.5)
            closing = writer.is_closing()
            extra_info = writer.get_extra_info('peername')
            if not closing and extra_info:
                kwargs.update({"result": True, "msg": "idc exists ok"})
                return [idc, *args], kwargs
            else:
                msg_error = "Closing %s, extra_info %s" % (
                    closing, extra_info)
                self.report("heartbeat", "msgerror "+msg_error)
                self.logger.error(msg_error)
                self.logger.error("no heart_beat, at %s" % tnow)
                print("IDC To close", idc)
                kwargs['result'] = False
                try:
                    await self.close(idc)
                    self.on_new_client(client_name)
                    print("Conexiones activas", self.active_conn)
                except BrokenPipeError as be:
                    self.report(
                        "heart_beat", "Close->Broken Pipe Error al cerrar %s bytes" % (be))
                    self.logger.exception(
                        f"Socket error {be}, mode {self.mode}")
                    await asyncio.sleep(1)
                    self.on_new_client(client_name)
                    await asyncio.sleep(2)
                except socket.error as e:
                    self.report(
                        "heart_beat", "Socket error, <%s>, client" % (e, client_name))
                    self.logger.exception(
                        "Socket error %s, mode %s" % (e, self.mode))
                    await asyncio.sleep(1)
                    self.on_new_client(client_name)
                    await asyncio.sleep(2)
                except asyncio.TimeoutError as te:
                    self.logger.exception(
                        "Tiempo fuera en intento de cerrar conexión %s, mode %s" % (te, self.mode))
                    self.on_new_client(client_name)
                    await asyncio.sleep(2)
                except (ConnectionResetError, ConnectionAbortedError) as conn_error:
                    self.logger.exception(
                        "Excepción por desconexión %s, mode %s" % (conn_error, self.mode))
                    self.on_new_client(client_name)
                    await asyncio.sleep(2)
                except Exception as e:
                    self.report(
                        "heart_beat", "Excepción no considerada, <%s>" % e)
                    self.logger.exception(
                        "Excepción no considerada %s, mode %s" % (e, self.mode))
                    await asyncio.sleep(1)
                    self.on_new_client(client_name)
                    await asyncio.sleep(2)
                self.logger.exception(
                    "Cerrando correctamente la conexión, por no heartbeat")
                self.status = False
                kwargs.update({"msg": "idc exists ok"})
                return [idc, *args], kwargs
        else:
            self.logger.error(
                "no heart_beat, idc %s not client,  mode %s" % (idc, self.mode))
            return [idc, *args], kwargs

    async def readbytes(self, reader, n, idc, origin="none"):
        future = reader.readexactly(n)
        try:
            result = await future
            return result
        except BrokenPipeError as be:
            self.report(
                "readbytes", "Close->Broken Pipe Error al cerrar %s bytes" % (be))
            await asyncio.sleep(5)
            return b''
        except socket.error as se:
            self.report(
                "readbytes", "Close->Socket Error al leer %s bytes" % (se))
            await asyncio.sleep(5)
            return b''
        except asyncio.TimeoutError as te:
            self.report("readbytes", "Error Timeout al leer %d bytes en %s, origen: %s" % (
                n, self.mode, origin))
            self.logger.exception("Mode %s, Tiempo fuera al leer en readbytes, %s, origen %s" % (self.mode, te,
                                                                                                 origin))
            await asyncio.sleep(5)
            return b''
        except asyncio.IncompleteReadError as ir:
            self.report("readbytes", "Incomplete read, mode %s" %
                        self.mode, reader, n)
            self.logger.exception(
                "Mode  %s, Tiempo fuera al no poder leer en readbytes %s bytes, %s" % (n, self.mode, ir))
            await asyncio.sleep(5)
            return b''
        except (ConnectionResetError, ConnectionAbortedError) as conn_error:
            self.logger.exception(
                "Excepción por desconexión al intentar leer %s" % conn_error)
            await asyncio.sleep(5)
            return b''
        except Exception as e:
            print(self.clients.keys())
            self.report("readbytes", "Excepción no considerada",
                        e, "client", idc, "origin", origin)
            self.logger.exception(
                "Excepción no considerada al intentar leer %s, %s" % (e, origin))
            await asyncio.sleep(5)
            rprint("cancelando llamada")
            return "CANCEL"

    async def recv_msg(self, id_client):
        if id_client in self.clients:
            reader = self.clients[id_client]['reader']
            writer = self.clients[id_client]['writer']
            # heartbeat = await self.heart_beat(id_client)
            # addr = self.addr
            bs_0 = 1
            count = 0
            b_header = b''
            idx_recv = ''
            t = True
            msg_tot = b''
            mlist = []
            n_msgs_idx = 0
            try:
                # moment 1 -> get hueader
                if writer.transport.is_closing():
                    # self.logger.error("La conexión se cerró %s" % writer)
                    raise Exception("Conexión perdida")
                idx_recv = ''
                page = [0, 0]
                while t:
                    while count < 4:
                        char_recv = b''
                        if not reader.at_eof():
                            # moment 2 -> get content message, check if there are more
                            try:
                                task_rb = self.readbytes(reader,
                                                         bs_0,
                                                         id_client,
                                                         origin="recv_msg_header")
                                char_recv = await task_rb
                                if char_recv == 'CANCEL':
                                    bprint(
                                        "Cancelando lectura de socket,reding header...")
                                    return ""
                                elif char_recv:
                                    b_header += char_recv
                            except asyncio.TimeoutError as te:
                                self.logger.exception("Tiempo fuera en intento de cerrar conexión %s, mode %s" % (
                                    te, self.mode))
                                await asyncio.sleep(10)
                                continue
                            except (ConnectionResetError, ConnectionAbortedError) as conn_error:
                                self.logger.exception("Excepción por desconexión %s, mode %s" % (
                                    conn_error, self.mode))
                                await asyncio.sleep(10)
                                continue
                            except Exception as ex:
                                self.report(
                                    "recv_msg", "Error al leer data -> %s" % ex, char_recv)
                                continue
                        else:
                            # buffer is empty
                            await asyncio.sleep(1)
                            return "{}"
                        # cuando ocurre un espacio, sumar count
                        # de otra manera continuar iteración
                        if char_recv == b" ":
                            count += 1
                    header = b_header.decode(char_code)
                    b_header = b''
                    spaces = [x.start() for x in re.finditer(' ', header)]
                    sp_1 = spaces[0]
                    sp_2 = spaces[1]
                    sp_3 = spaces[2]
                    # obtener número de página
                    s_page = header[sp_2 + 1:sp_3].split("/")
                    a = page[0]
                    b = page[1]
                    page = list(map(hextring2int, s_page))
                    sp_4 = spaces[3]
                    check_header = header[sp_1 + 1:sp_2]
                    assert self.header == header[sp_1 + 1:
                                                 sp_2], "No es un encabezado correcto"
                    this_idx_recv = header[:sp_1]
                    if idx_recv == '':
                        idx_recv = this_idx_recv
                        n_msgs_idx = 1
                    else:
                        n_msgs_idx += 1
                    assert n_msgs_idx == page[
                        0], "Error en mensaje, no coincide #idx con pagina " + str(
                            page[0])
                    lmsg = header[sp_3 + 1:sp_4]
                    bs_1 = hextring2int(lmsg)+1
                    b_MSG = b''
                    n = 0
                    while n < bs_1:
                        bm = None
                        # moment 2 -> get content message, check if there are more
                        try:
                            bm = await self.readbytes(
                                reader, 1, id_client, origin="recv_msg_body")
                            if bm == 'CANCEL':
                                return ""
                        except asyncio.TimeoutError as te:
                            self.logger.exception("Tiempo fuera en intento de cerrar conexión %s, mode %s" % (
                                te, self.mode))
                            await asyncio.sleep(10)
                            continue
                        except (ConnectionResetError, ConnectionAbortedError) as conn_error:
                            self.logger.exception("Excepción por desconexión %s, mode %s" % (
                                conn_error, self.mode))
                            await asyncio.sleep(10)
                            continue
                        except Exception as ex:
                            self.logger.exception(
                                "Error al leer datos %s, con %s" % (ex, b_MSG))
                            print("Error en conexión excepction:<%s>" % ex)
                            print("Mensaje....", bm)
                            continue
                        if bm:
                            b_MSG += bm
                            n += 1
                    l_ender = len(self.ender.encode(char_code))
                    nlast = b_MSG.rfind(b" ")
                    pre_msg = b_MSG[:nlast]
                    count = 0
                    if pre_msg == b"<END>" or page[0] == page[1]:
                        t = False
                        mlist.append(pre_msg)
                        n_msgs_idx = 0
                        break
                    else:
                        mlist.append(pre_msg)
            except asyncio.CancelledError as ex:
                self.logger.exception("Error de conexión cancelada  %s" % ex)
                self.report(
                    "recv_msg", "Asyncio cancelación de conexión <%s>" % ex)
                self.queue_cancel.put(id_client)
            except socket.error as ex:
                self.logger.exception(f"Error de conexión de socket  {ex}")
                self.report("recv_msg", "Error de socket")
                await asyncio.sleep(3)
                self.report(
                    "recv_msg", "Falla en la conexión al recibir mensaje, %s" %
                    ex)
                self.queue_cancel.put(id_client)
            except Exception as ex:
                await asyncio.sleep(3)
                self.logger.exception("Error de conexión  %s" % ex)
                self.report(
                    "recv_msg", "Falla en la conexión al recibir mensaje, %s" % ex)
                self.report("recv_msg", "Lista msgs", mlist)
            #print("Procesando final")
            msg_tot = b"".join(mlist)
            MSG_TOT = msg_tot.decode(char_code)
            MT = MSG_TOT.strip()
            MT = MSG_TOT.rstrip()
            self.msg_r = MSG_TOT
            return MSG_TOT
        else:
            return "{}"

    def get_path(self):
        return self.gnc_path

    def get_address(self):
        return self.address

    def set_address(self, address):
        self.address = address

    def set_path(self, new_gnc_path):
        if os.path.exists(new_gnc_path):
            os.remove(new_gnc_path)
        self.gnc_path = new_gnc_path

    # callback for create server:

    def set_idc(self):
        """
        Defines a new id for relation process-collect_task, check if exists
        """
        uin = 4
        idc = my_random_string(uin)
        while True:
            if idc not in self.idc:
                self.idc.append(idc)
                break
            else:
                idc = my_random_string(uin)
        return idc

    async def set_reader_writer(self, reader, writer):
        idc = self.set_idc()
        # self.log_info_client(writer)
        new_client = {
            'reader': reader,
            'writer': writer
        }
        self.clients[idc] = new_client
        return idc

    def log_info_client(self, writer):
        names = ['peername', 'socket', 'sockname']
        for name in names:
            info = writer.get_extra_info(name)
            # self.loggger.info("La info correspondiente a %s es %s" %(name, info))

    def off_blocking(self):
        if self.mode == 'server':
            [sock.setblocking(False) for sock in self.server.sockets]
        elif self.mode == 'client':
            pass

    def on_blocking(self):
        if self.mode == 'server':
            [sock.setblocking(True) for sock in self.server.sockets]
        elif self.mode == 'client':
            pass

    def settimeout(self, timeout=5):
        [sock.settimeout(timeout) for sock in self.server.sockets]

    async def connect(self):
        """
        Connect client to socket
        """
        while not self.status:
            try:
                result = await self.loop.sock_connect(
                    self.sock, self.address)
                self.alert = "Conectado a socket-base"
                self.status = True
                self.report("async connect", "Resultado de conectar socket")
                print(result)
                # self.logger.info("Conexión a %s realizada" % self.address)
                break
            except socket.timeout as timeout:
                # self.on_blocking()
                # self.logger.error(
                #     "Error en conexión con %s, error: %s" % (self.address, timeout))
                self.report("async connect", "Error de socket a GSOF \
                en conexión %s, %s" % (self.address, timeout))
                self.status = False
                await asyncio.sleep(.5)
                await self.connect()  # go to begin -|^
            except socket.error as e:
                self.status = False
                # self.on_blocking()
                if e.errno == errno.ECONNREFUSED:
                    pass
                    # self.logger.error(e)
                else:
                    pass
                    # self.logger.error(e)

    def clean_socket(self, host, port):
        comSocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.report("clean socket", "ex socket->ComSocket:::", comSocket)
        self.report("clean socket", socket.SOL_SOCKET, socket.SO_REUSEADDR)
        self.report("clean socket", socket.SOL_SOCKET, socket.SO_REUSEPORT)
        self.report("clean socket", "Cleaning address", (host, port))
        comSocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        comSocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
        # comSocket.shutdown(socket.SHUT_RDWR)

    async def close(self, idc):
        self.closing[idc] = True
        self.report("close", "La conexión se cerró en cliente %s" % idc)
        self.logger.error("La conexión se cerró en cliente %s" % idc)
        self.status = False
        print(idc, self.clients)
        if idc in self.clients:
            reader = self.clients.get(idc, {}).get('reader')
            writer = self.clients.get(idc, {}).get('writer')
            try:
                if writer:
                    if writer.can_write_eof():
                        print("Closing idc client,,,,")
                        writer.close()
                del self.clients[idc]
                del self.closing[idc]
                self.drop_conn(idc)
                print("Deleted idc de clients", idc, self.clients)
            except BrokenPipeError as be:
                self.report(
                    "close", "Close->Broken Pipe Error al cerrar %s bytes" %
                    (be))
                self.queue_cancel.put(idc)
                raise be
            except socket.error as se:
                self.report(
                    "close", "Close->Socket Error al leer %s bytes" % (se))
                self.queue_cancel.put(idc)
                raise se
            except asyncio.TimeoutError as te:
                self.report(
                    "close", "Close->Error Timeout al leer %d bytes" % (te))
                self.logger.exception(
                    "Tiempo fuera al leer en close, %s" % (te))
            except asyncio.IncompleteReadError as ir:
                self.report(
                    "close", "Close_>Incomplete read, reader %s" % reader)
                self.logger.exception(
                    "Station %s, Tiempo fuera al no poder leer en readbytes %s bytes, %s" % (n, self.station, ir))
            except (ConnectionResetError, ConnectionAbortedError) as conn_error:
                self.logger.exception(
                    "Close->Excepción por desconexión al intentar leer %s" %
                    conn_error)
                self.queue_cancel.put(idc)
                raise conn_error
            except Exception as e:
                self.report("close", "Excepción no considerada", e)
                self.logger.exception(
                    "Close->Excepción no considerada al intentar leer%s" % e)
                raise e
        else:
            raise Exception("No existe %s" % idc)

    async def create_server(self, callback_io, loop):
        # SI ES UNIX SOCKET
        # includes  bind
        # https://github.com/python/asyncio/blob/fff05d480760703adbc3e2d4cb3dbcfbff803c29/asyncio/unix_events.py
        bprint("loop runing before create server")
        self.report("create server", loop.is_running())
        mode = self.mode
        context = self.context
        self.report("create server", "Creando server---_>")
        self.report("create server", loop, mode, context, AF_TYPE)
        self.server = None
        try:
            if mode == 'server':
                if AF_TYPE == socket.AF_UNIX:
                    self.report("create server",  "Se crea Socket Server Unix")
                    # ref https://docs.python.org/3/library/asyncio-eventloop.html
                    future_server = asyncio.start_unix_server(callback_io,
                                                              loop=loop,
                                                              path=self.get_path(),
                                                              limit=self.backlog,
                                                              ssl=context)
                    self.server = await asyncio.wait_for(
                        future_server, timeout=self.timeout)
                elif AF_TYPE in {socket.AF_INET, socket.AF_INET6}:
                    host = self.address[0]
                    port = self.address[1]
                    self.report("create server", "Se crea Socket Server TCPx")
                    self.report("create server", "Cleaning address")
                    self.clean_socket(host, port)
                    self.report("create server", "Closing cleaning address")
                    self.report("create server", "New server listening......")
                    rprint(host)
                    rprint(port)
                    future_server = asyncio.start_server(
                        callback_io,
                        loop=loop,
                        host=host,
                        port=port,
                        family=AF_TYPE,
                        backlog=self.backlog,
                        ssl=context)
                    self.report("create server",
                                "Future coro to run server:::")
                    rprint(future_server)
                    self.server = await asyncio.wait_for(
                        future_server, timeout=self.timeout)
                    self.report("create server",
                                "(create_server works!)El server es")
                    self.report("create server", loop)
                    print(self.server, type(self.server))
            else:
                self.report("create server", "Asigna primero mode=server")
        except asyncio.TimeoutError as te:
            self.logger.exception("Tiempo fuera en intento de cerrar conexión %s, mode %s" % (
                te, self.mode))

            await asyncio.sleep(10)
            raise te
        except (ConnectionResetError, ConnectionAbortedError) as conn_error:
            self.logger.exception("Excepción por desconexión %s, mode %s" % (
                conn_error, self.mode))
            self.report("create server",
                        "Excepcion error de conexión", conn_error)
            await asyncio.sleep(10)
        except Exception as ex:
            self.report("create server",
                        "Excepcion en create_server <::::", ex, "::::>")
        except asyncio.TimeoutError as te:
            self.report("create server", "Timeout error", te)
            raise te
        self.status = 'OFF'
        return self.server

    async def create_client(self):
        mode = self.mode
        loop = self.loop
        self.send_msg_sock_status(20)
        reader, writer = (None, None)
        idc = ""
        while self.status != 'ON':
            try:
                if mode == 'client':
                    if AF_TYPE == socket.AF_UNIX:
                        # ref https://docs.python.org/3/library/asyncio-eventloop.html
                        future_unix_client = asyncio.open_unix_connection(
                            loop=loop, path=self.get_path(), ssl=self.context)
                        (reader, writer) = await asyncio.wait_for(future_unix_client, timeout=self.timeout)
                        self.send_msg_sock_status(40)
                    elif AF_TYPE == socket.AF_INET or AF_TYPE == socket.AF_INET6:
                        host = self.address[0]
                        port = self.address[1]
                        print("La direccion es %s con tipo: %s " % (self.address,
                                                                    AF_TYPE))
                        future_client = asyncio.open_connection(
                            loop=loop,
                            host=host,
                            port=port,
                            ssl=self.context)
                        (reader, writer) = await asyncio.wait_for(future_client, timeout=self.timeout)
                        self.send_msg_sock_status(40)
                    idc = await self.set_reader_writer(reader, writer)
                    self.report("create client", "Cliente creado--->", idc)
                    self.report("create client", "Clientes", self.clients)
                    self.send_msg_sock_status(50)
                    self.alert = "Conectado a socket-base"
                    self.status = 'ON'
                    self.report("create client", self.alert)
                    self.report("create client", "Creando nuevo client")
            except asyncio.TimeoutError as te:
                self.logger.exception("Tiempo fuera en intento de cerrar conexión %s, mode %s" % (
                    te, self.mode))
                self.report("create server", "Excepcion error de timeout", te)
                self.status = 'OFF'
                await asyncio.sleep(10)
            except (ConnectionResetError, ConnectionAbortedError) as conn_error:
                self.logger.exception("Excepción por desconexión %s, mode %s" % (
                    conn_error, self.mode))
                await asyncio.sleep(10)
                self.status = 'OFF'
            except Exception as ex:
                self.report("create client", "Excepcion en", ex)
                print("Creating new client?", self.new_client)
                self.send_msg_sock_status(-1)
                self.status = 'OFF'
                await asyncio.sleep(10)
        return idc

    def set_backlog(self, new_backlog):
        # ?
        assert isinstance(new_backlog,
                          int), "El nuevo backlog no es un valor válido"
        self.backlog = new_backlog

    async def accept(self):
        conn, addr = await self.loop.sock_accept(self.server.sockets[0])
        self.conns.append(conn)
        self.addrs.append(addr)
        self.conn = conn
        self.addr = addr
        self.status = 'ON'
        return conn, addr

    def list_clients(self):
        for i in range(len(self.conss)):
            print(str(self.addrs[i]) + ":" + str(self.conns[i]))

    def clean_client(self, idc):
        self.report("clean_client", "Limpiando cliente", idc)
        if idc in self.clients:
            client = self.clients.get(idc)
            client.get('writer').close()
            del self.clients[idc]

    async def wait_closed(self):
        for ids, client in self.clients.items():
            await client.get('writer').wait_closed()
        if self.mode == 'server' and self.server:
            await self.server.wait_closed()

    async def server(self):
        await self.create_server()

    async def client(self):
        return await self.create_client()
        # Connect to path

    def get_name(self):
        return self.gnc_path

    def __enter__(self):
        self.report("enter socket", "Starting GNC Socket (enter)")
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        bprint("="*20)
        self.report("enter socket", "Enviando EOF al otro lado")
        self.report("enter socket", "Clossing succesful, Socket exit->",
                    exception_type, exception_value, traceback)
        self.report("enter socket", "Socket exit -> traceback", traceback)
        self.report("enter socket", "Closing GNC Socket")
        # self.close()
        self.report("enter socket", "Socket closed")
        self.report("enter socket", self.loop)
        self.report("enter socket", "Loop status |^")
        # self.loop.run_until_complete(self.wait_closed())
        self.report("enter socket", "="*20)



import socket
from basic_queuetools.queue import read_queue_gen
from gnsocket.gn_socket import GNCSocket
# Standar lib
import asyncio
import functools
from multiprocessing import Manager, Queue, Lock

# contrib modules
import ujson as json

# Own module
from gnsocket.gn_socket import GNCSocket
from gnsocket.socket_base import GNCSocketBase

# module tasktools
from networktools.colorprint import gprint, bprint, rprint

from networktools.library import pattern_value, \
    fill_pattern, context_split, \
    gns_loads, gns_dumps
from networktools.library import my_random_string
from asyncio import shield, wait_for, wait

import ujson as json
from tasktools.taskloop import TaskLoop

tsleep = 2


class GNCSocketClient(GNCSocketBase):

    def __init__(self, queue_n2t, queue_t2n, *args, **kwargs):
        super().__init__(queue_n2t, queue_t2n, 'client', *args, **kwargs)
        self.set_socket_task(self.socket_task)
        self.name = kwargs.get('name', "test")
        self.idc = None

    def socket_task(self):
        # client socket
        loop = asyncio.get_event_loop()
        self.connected = False
        with GNCSocket(mode='client', timeout=self.timeout,
                       raise_timeout=self.raise_timeout,
                       log_path=self.log_path) as gs:
            try:
                gs.set_new_client(self.client_name)
                self.loop = loop
                gs.set_address(self.address)
                gs.set_loop(loop)

                async def client_cycle(*args, **kwargs):
                    if gs.new_client(self.client_name):
                        try:
                            gs.report("client_cycle", "Creating new client...")
                            idc = await gs.create_client()
                            self.new_client("client", self.name, idc)
                            self.idc = idc
                            self.queue_channel(idc)
                            self.stop_read[idc] = False
                            self.stop_write[idc] = False
                            gs.report("client_cycle", "idc->%s" % idc)
                            await gs.new_client_queue.put({
                                "idc": idc,
                            })
                            gs.relation[self.client_name] = idc
                            gs.off_new_client(self.client_name)
                            print("Post connection", gs.active_conn)
                            # send client info to socket server
                            client_info = {
                                "SOCKET_COMMAND": "NEW_CLIENT",
                                "value": idc,
                                "name": self.client_name,
                                "idc": idc
                            }
                            self.qn2t.put(client_info)
                            print(f"Sended {client_info}")
                            await asyncio.sleep(2)
                            #
                            return [False, idc, *args], kwargs
                        except asyncio.CancelledError as ce:
                            gs.report("client_cycle",
                                      "Tareas canceladas...", ce)
                            gs.logger.exception("Tareas canceladas %s" % ce)
                            if idc in gs.clients.keys():
                                gs.clients.remove(idc)
                            gs.on_client_name()
                            return [True, *args], {}
                        except asyncio.TimeoutError as te:
                            gs.report("client_cycle",
                                      "Error timeout...", te, gs.mode)
                            gs.logger.exception("Tiempo fuera en escritura %s, mode %s" % (
                                te, gs.mode))
                            if idc in gs.clients.keys():
                                gs.clients.remove(idc)
                            await asyncio.sleep(10)
                            gs.on_client_name()
                            return [True, *args], {}
                        except (ConnectionResetError, ConnectionAbortedError) as conn_error:
                            gs.report("client_cycle",
                                      "Error conexion...", conn_error, gs.mode)
                            gs.logger.exception("Excepción por desconexión %s, mode %s" % (
                                conn_error, gs.mode))
                            await asyncio.sleep(10)
                            if idc in gs.clients.keys():
                                gs.clients.remove(idc)
                            gs.on_client_name()
                            return [True, *args], {}
                        except Exception as e:
                            gs.report("client_cycle",
                                      "Excepción no considerada", e)
                            gs.logger.exception(
                                "Excepción no considerada al intentar leer%s" % e)
                            gs.on_client_name()
                            await asyncio.sleep(5)
                            return [True, *args], {}
                    else:
                        await asyncio.sleep(2)
                        return args, kwargs

                async def socket_io():
                    idc = None
                    try:
                        # client manage control
                        create_client = True
                        args = [create_client]
                        task_client_cycle = TaskLoop(client_cycle,
                                                     args, **{
                                                         "name": "task_client_cycle"
                                                     })
                        # client heartbeat
                        args = [None]
                        task_heart_beat = TaskLoop(gs.heart_beat,
                                                   args,
                                                   {"client_name": self.client_name,
                                                    "sleep": 5},
                                                   **{"name": "task_heart_beat"})
                        # sock_read
                        args = [gs, idc]

                        task_sock_read = TaskLoop(
                            self.sock_read,
                            args,
                            {},
                            **{"name": "task_sock_read"}
                        )

                        args = [gs, idc]
                        # task write
                        task_sock_write = TaskLoop(
                            self.sock_write,
                            args,
                            {},
                            **{"name": "task_sock_write"}
                        )

                        task_from = TaskLoop(
                            self.from_socket, [], {}, **{"name": "from_socket"})
                        task_to = TaskLoop(
                            self.to_socket, [], {}, **{"name": "to_socket"})

                        # create taskloops
                        tasks = [
                            task_client_cycle,
                            task_heart_beat,
                            task_sock_read,
                            task_sock_write,
                            task_from,
                            task_to
                        ]
                        for task in tasks:
                            task.create()
                    except Exception as ex:
                        gs.abort(idc)
                        await gs.close(idc)
                        gs.logger.exception(
                            "Cancelacion de tareas en gnsocket, error %s" % ex)
                        task_sock_write.close()
                        task_sock_read.close()
                        return False, {}
            except Exception as ex:
                gs.logger.exception(
                    "Error con modulo cliente gnsocket %s" % ex)
                gs.report("socket_task", "Exception as %s" % ex)
                raise ex
            # run the tasks :)
            future1 = loop.create_task(socket_io())
            if not loop.is_running():
                loop.run_forever()


if __name__ == "__main__":
    address = (socket.gethostbyname(socket.gethostname()), 5500)
    client = GNCSocketClient(address=address)
    client.socket_task()



import asyncio
from gnsocket.gn_socket import GNCSocket
from gnsocket.conf.socket_conf import TEST_TEXT
from tasktools.taskloop import coromask, renew, simple_fargs
import functools
from termcolor import colored, cprint

def gprint(text):
    msg = colored(text, 'green', attrs=['reverse', 'blink'])
    print(msg)

def bprint(text):
    msg = colored(text, 'blue', attrs=['reverse', 'blink'])
    print(msg)

def rprint(text):
    msg = colored(text, 'red', attrs=['reverse', 'blink'])
    print(msg)

if __name__ == "__main__":
    mode = 'server'
    gs = GNCSocket(mode=mode)
    print(gs.gnc_path)
    msg = TEST_TEXT

    # Testing message generator
    print("Status " + gs.status)
    for m in gs.generate_msg(msg):
        print(m.decode('utf-8'))
    # gs.send_msg(msg)
    # gs.send_msg(msg)
    # print(gs.conn)
    # Testing communicate
    print(gs)
    print("Entrando a loop")
    loop = asyncio.get_event_loop()
    gs.set_loop(loop)
    tsleep=1# 1 second sleep by some coroutines
    # Create coroutines
    async def sock_read(queue):
        bprint("Sock read")
        try:
            datagram = await gs.recv_msg()
            bprint("msg recibido")
            if not datagram == '' and \
               datagram != "<END>":
                await queue.put(datagram)
            await asyncio.sleep(tsleep)
            # print(msg_tot)
        except Exception as exec:
            gs.set_status('OFF')
            raise exec

    async def sock_write(queue):
        rprint("Sock write")
        print("Connection %s " % format(gs.writer.transport.is_closing()))
        #read async queue
        try:
            if not queue.empty():
                for q in range(queue.qsize()):
                    msg=await queue.get()
                    await gs.send_msg(msg)
                await gs.send_msg("<END>")
                rprint("Msg enviado")
            await asyncio.sleep(tsleep)
        except Exception as exec:
            gs.set_status('OFF')
            raise exec

    async def socket_io(reader, writer):
        queue=asyncio.Queue()
        await gs.set_reader_writer(reader, writer)
        #First time welcome
        welcome="Welcome to socket"
        rprint(welcome)
        await gs.send_msg(welcome)
        await gs.send_msg("<END>")
        #task reader
        try:
            args=[queue]
            task=loop.create_task(
                coromask(
                    sock_read,
                    args,
                    simple_fargs)
            )
            task.add_done_callback(
                functools.partial(
                    renew,
                    task,
                    sock_read,
                    simple_fargs)
            )
            #task write
            task=loop.create_task(
                coromask(
                    sock_write,
                    args,
                    simple_fargs)
            )
            task.add_done_callback(
                functools.partial(
                    renew,
                    task,
                    sock_write,
                    simple_fargs)
            )
        except Exception as exec:
            raise exec

    try:
        future=loop.create_task(gs.create_server(socket_io))
        loop.run_forever()
    except:
        future.exception()
        raise
    
    loop.close()
    gs.close()



from setuptools import setup, find_packages

from pathlib import Path

path = Path(__file__).resolve().parent
with open(path/'README.md', encoding='utf-8') as f:
    long_description = f.read()

with open(path/'VERSION') as version_file:
    version = version_file.read().strip()


setup(name='gnsocket',
      version=version,
      description='GPS Network Socket, with asyncio stream manager',
      url='https://gitlab.com/pineiden/gus',
      author='David Pineda Osorio',
      author_email='dpineda@csn.uchile.cl',
      license='GPLv3',
      install_requires=[
          "ujson",
          "networktools",
          "basic_queuetools",
          "basic_logtools",
          'chardet'],
      packages=["gnsocket"],      
      include_package_data=True,      
      package_dir={'gnsocket': 'gnsocket'},
      python_requires='>=3.7',
      long_description=long_description,
      long_description_content_type='text/markdown',
      zip_safe=False)



# Standar lib
from asyncio import wait_for
import asyncio

# contrib modules
import ujson as json

# Own module
from gnsocket.gn_socket import GNCSocket
from gnsocket.socket_base import GNCSocketBase

# module tasktools
from tasktools.taskloop import TaskLoop
from networktools.colorprint import gprint, bprint, rprint

tsleep = 2


class GNCSocketServer(GNCSocketBase):

    def __init__(self, queue_n2t, queue_t2n, *args, **kwargs):
        super().__init__(queue_n2t, queue_t2n, 'server', *args, **kwargs)
        self.set_socket_task(self.socket_task)
        self.commands = {
            'NEW_CLIENT': self.new_client
        }
        self.server = {}
        self.main_tasks = {}
        self.stop_check = False

    def socket_task(self):
        print("Socket task...")
        with GNCSocket(mode=self.mode,
                       timeout=self.timeout,
                       raise_timeout=self.raise_timeout,
                       log_path=self.log_path) as gs:
            print("Iniciando Socket Task en modo %s" % self.mode)
            loop = asyncio.get_event_loop()
            self.loop = loop
            gs.set_address(self.address)
            gs.set_loop(loop)
            try:
                async def socket_io(reader, writer):
                    try:
                        idc = await gs.set_reader_writer(reader, writer)
                        self.client = idc
                        bprint("Creando canal de colas para cliente")
                        self.queue_channel(idc)
                        rprint(self.queues[idc])
                        gs.reading[idc] = False
                        self.stop_read[idc] = False
                        self.stop_write[idc] = False
                        self.server[idc] = asyncio.current_task()
                        gs.set_new_client(idc)
                        # First time welcome
                        welcome = json.dumps(
                            {"msg": "Welcome to socket", 'idc_server': idc})
                        await wait_for(gs.send_msg(welcome, idc), timeout=10)
                        await asyncio.sleep(0.1)
                        # task reade
                    except asyncio.TimeoutError as te:
                        gs.report(
                            "socket_task", f"Tiempo fuera en escritura {te}, mode {gs.mode}")
                        gs.logger.exception(
                            f"Tiempo fuera en escritura {te}, mode {gs.mode}")
                        await asyncio.sleep(10)
                    except (ConnectionResetError,
                            ConnectionAbortedError) as conn_error:
                        gs.report(
                            "socket_task", f"Excepción por desconexión {conn_error}, mode {gs.mode}")
                        gs.logger.exception(
                            f"Excepción por desconexión {conn_error}, mode {gs.mode}")
                        await asyncio.sleep(10)
                    try:
                        args = [gs, idc]
                        task_name = f"sock_read_{idc}"
                        task_sock_read = TaskLoop(self.sock_read, args,
                                                  **{
                                                      "name": task_name
                                                  })
                        task_sock_read.create()
                        self.main_tasks[task_name] = task_sock_read
                        # task write
                        task_name = f"sock_write_{idc}"
                        task_sock_write = TaskLoop(self.sock_write, args,
                                                   **{
                                                       "name": task_name
                                                   })
                        self.main_tasks[task_name] = task_sock_write
                        task_sock_write.create()
                        # check_connection(self, idc, gs, *args, **kwargs
                    except Exception as exe:
                        raise exe
                future = loop.create_task(
                    gs.create_server(socket_io, loop),)

                # activar tarea de control de conexión
                args = [gs, None]

                task_check = TaskLoop(self.check_connection, args,
                                      {},
                                      **{"name": "check_connection"})
                task_check.create()
                bprint(f"Task check- > {task_check}")

                task_from = TaskLoop(self.from_socket, [], {},
                                     **{"name": "from_socket"})
                task_to = TaskLoop(self.to_socket, [], {},
                                   **{"name": "to_socket"})

                task_from.create()
                task_to.create()

                if not loop.is_running():
                    loop.run_forever()
                else:
                    loop.run_until_complete(future)
            except KeyboardInterrupt as k:
                gs.report("socket_task", "Closing with keyboard")
                gs.logger.exception(f"Cierre forzado desde teclado {k}")
                loop.run_until_complete(gs.wait_closed())
                raise k
            except Exception as ex:
                gs.report("socket_task", "Exception", ex)
                gs.logger.exception(
                    "Error con modulo cliente gnsocket %s" % ex)
                print("Otra exception", ex)

    def close(self):
        loop = asyncio.get_event_loop()
        loop.close()

    async def check_connection(self, gs, idc, *args, **kwargs):
        cancel_set = set()
        if not gs.queue_cancel.empty():
            for i in range(gs.queue_cancel.qsize()):
                idserver = gs.queue_cancel.get()
                cancel_set.add(idserver)
        for client_name, collection in self.clients.items():
            for socket_id in list(collection.keys()):
                try:
                    hb_args, hb_kwargs = await gs.heart_beat(socket_id)
                    if not hb_kwargs.get('result') or socket_id in cancel_set:
                        await self.drop_client(
                            client_name,
                            socket_id,
                            gs)
                except Exception as e:
                    print("Heartbeat exception on check_connection")
                    raise e
        await asyncio.sleep(3)
        return [gs, idc, *args], kwargs

    async def drop_client(self, client_name, socket_id, gs):
        if socket_id in self.clients[client_name]:
            # close tasks
            self.stop_read[socket_id] = True
            self.stop_write[socket_id] = True
            selected = [f"task_sock_write_{socket_id}"
                        f"task_sock_read_{socket_id}"]
            for name in selected:
                for task in self.main_tasks.values():
                    print("Task alive", task.name)
                    print(
                        f"Cancelled {task.cancelled()} or done {task.done()}")
                    if name == task.name:
                        rprint(f"Cancelando tarea {task.get_name}")
                        task.stop()
                        await asyncio.sleep(.5)
                        task.cancel()
                        try:
                            task.result()
                        except asyncio.CancelledError as ce:
                            bprint(
                                f'''tarea {task.get_name()} cancelada
                                exitosamente {ce}''')
            server_task = self.server.get(socket_id)
            if server_task:
                server_task.cancel()
                try:
                    server_task.result()
                except asyncio.CancelledError as ce:
                    bprint(
                        f'''tarea {server_task.get_name()}
                        cancelada exitosamente {ce}''')
                    bprint(
                        f'''tarea {server_task.get_name()}
                        cancelada exitosamente {ce}''')
                    bprint(f"Is server canceled? {server_task.cancelled()}")
                del self.server[socket_id]
            del self.clients[client_name][socket_id]
            rprint(f"Ids en cliente: {self.clients[client_name]}")
